<?php
/**
 * ShopEx licence
 *
 * @copyright  Copyright (c) 2005-2010 ShopEx Technologies Inc. (http://www.shopex.cn)
 * @license  http://ecos.shopex.cn/ ShopEx License
 */

// http://localhost/bbc/public/index.php/utils/apiCheck.html?api=trade.order.list.get
class toputil_ctl_apiCheck {

    public function __construct()
    {
        if (!config::get('app.debug'))
        {
            return kernel::abort(404);
        }

        pamAccount::setAuthType('desktop');
        if( ! pamAccount::getAccountId() )
        {
            $url = url::route('shopadmin');
            echo "<script>location ='$url'</script>";exit;
        }

        if( ! kernel::single('desktop_user')->is_super() )
        {
            return kernel::abort(404);
        }
    }

    private function __common()
    {
        $apis = config::get('apis.routes');
        foreach ($apis as $key => &$value)
        {
            list($class, $method) = explode('@', $value['uses']);
            $handlar = new $class;
            $value['apidesc'] = $this->__getTitle1($handlar);
            $apigroup = $this->__getTitle2($class);
            $pagedata['list'][$apigroup][$key] = $value;
            $pagedata['count'][$apigroup] += 1;
        }

        $pagedata['apis'] = $apis;
        return $pagedata;
    }

    public function getApiDoc(){

        $pagedata = $this->__common();

        $apiKey = trim(input::get('api'));
        if($apiKey && $pagedata['apis'][$apiKey])
        {
            $apiConf = $pagedata['apis'][$apiKey];
            $handle = $apiConf['uses'];
            list($class, $method) = explode('@', $handle);
            $handlar = new $class;
            $pagedata['apiKey'] = $apiKey;
            $pagedata['apidesc'] = $apiConf['apidesc'];
            $pagedata['activeGroupName'] = $this->__getTitle2($handle);
            $pagedata['system_params'] = $this->__getSystemParams();
            $pagedata['params'] = $this->__getParams($handlar);
            $pagedata['response'] = $this->__getResponse($class, $method);

            $pagedata['output'] = 'toputil/apiCheck/info.html';
        }
        else
        {
            //指定的分类API列表
            if( input::get('group') && $pagedata['list'][input::get('group')] )
            {
                $pagedata['activeGroupList'] = $pagedata['list'][input::get('group')];
                $pagedata['activeGroupName'] = input::get('group');
            }
            else//默认第一个分类的API列表
            {
                $pagedata['activeGroupIsList'] = true;
                $pagedata['activeGroupList'] = current($pagedata['list']);
                $pagedata['activeGroupName'] = key($pagedata['list']);
            }

            $pagedata['output'] = 'toputil/apiCheck/list.html';
        }

        return view::make('toputil/apiCheck/index.html', $pagedata);
    }

    public function testApi()
    {
        $apiKey = trim(input::get('api'));

        $pagedata = $this->__common();

        if($apiKey && $pagedata['apis'][$apiKey])
        {
            $apiConf = $pagedata['apis'][$apiKey];
            $handle = $apiConf['uses'];
            list($class, $method) = explode('@', $handle);
            $handlar = new $class;
            $pagedata['apiKey'] = $apiKey;
            $pagedata['groupKey'] = explode('_',$class)[0];
            $pagedata['apidesc'] = $apiConf['apidesc'];
            $pagedata['activeGroupName'] = $this->__getTitle2($handle);
            $pagedata['system_params'] = $this->__getSystemParams();
            $pagedata['params'] = $this->__getParams($handlar);
        }

        //执行URL
        if( input::get('type') == 'use' && $_POST )
        {
            $url = kernel::base_url(1).kernel::url_prefix().'/api';

            $apiParams = input::get('params');
            $pagedata['apiParams'] = $apiParams;
            $apiParams['method'] = $apiKey;
            $apiParams['timestamp'] = time();
            $apiParams['sign_type'] = 'MD5';
            $apiParams['sign'] = base_rpc_validate::sign($apiParams,base_certificate::token());

            $runtimeStart = microtime(true);
            $result = client::post($url, ['body' => $apiParams])->json();
            $runtimeStop= microtime(true);
            $runtime = round(($runtimeStop - $runtimeStart) , 4);

            $pagedata['apiRunTime'] = $runtime;
            $pagedata['result_json'] = json_encode($result);
            $pagedata['result'] = $result;
        }

        $pagedata['output'] = 'toputil/apiCheck/test.html';
        return view::make('toputil/apiCheck/index.html', $pagedata);
    }


    private function __getTitle2($handle)
    {
        $args = explode('_', $handle);
        $app_name = $args[0];
        $app_titles = config::get('prism.prismApiName', array());
        $app_title = $app_titles[$app_name] ? $app_titles[$app_name] : $app_name;

        return $app_title;
    }

    private function __getTitle1($api_handlar)
    {
        return $api_handlar->apiDescription;
    }

    private function __getSystemParams()
    {
        return [
            [
                'field' => 'format',
                'title' => '返回数据格式',
                'type' => 'string',
                'validate' => 'required',
                'example' => 'json',
                'desc' => '返回数据是json格式的，目前只支持json',
            ],
            [
                'field' => 'v',
                'title' => '版本号',
                'type' => 'string',
                'validate' => 'required',
                'example' => 'v1',
                'desc' => '标识该接口的版本',
            ],
        ];
    }

    private function __getParams($handlar)
    {
        $return = $handlar->getParams();
        $params = $return['params'];
        $ret = array();
        foreach($params as $key=>$value)
        {
            $field = array();
            $field['field'] = $key;
            $field['title'] = $value['title']?:$value['description'];
            $field['type'] = $value['type'];
            $field['validate'] = $value['valid'];
            $field['example'] = $value['example'];
            $field['desc'] = $value['desc']?:$value['description'];
            $ret[] = $field;
        }
        return $ret;
    }

    private function __getResponse($class, $method)
    {
        $rMethod = new ReflectionMethod($class, $method);

        $docComment = $rMethod->getDocComment();
        $docCommentArr = explode("\n", $docComment);

        foreach ($docCommentArr as $comment)
        {
            $comment = trim($comment);

            //标题描述
            if (empty($description) && strpos($comment, '@') === false && strpos($comment, '/') === false)
            {
                $description = substr($comment, strpos($comment, '*') + 1);
                continue;
            }

            //@desc注释
            $pos = stripos($comment, '@desc');
            if ($pos !== false)
            {
                $descComment = substr($comment, $pos + 5);
                continue;
            }

            //@return注释
            $pos = stripos($comment, '@return');
            if ($pos === false)
            {
                continue;
            }

            $returnCommentArr = explode(' ', substr($comment, $pos + 8));
            if (count($returnCommentArr) < 2)
            {
                continue;
            }
            if (!isset($returnCommentArr[2]))
            {
                $returnCommentArr[2] = '';  //可选的字段说明
            }
            else
            {
                //兼容处理有空格的注释
                $returnCommentArr[2] = implode(' ', array_slice($returnCommentArr, 2));
            }

            $returns[] = $returnCommentArr;
        }
        return $returns;
    }

}
